from __future__ import unicode_literals
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.utils.html import format_html
from django.db import models

@python_2_unicode_compatible
class Member(models.Model):
    name = models.CharField(_('name'), max_length=200)
    avatar = models.CharField(_('avatar'), max_length=200)
    openid = models.CharField(_('openid'), max_length=200)
    city = models.CharField(_('city'), max_length=200)
    created = models.DateTimeField(_('created time'), auto_now_add=True)
    updated = models.DateTimeField(_('updated time'), auto_now=True)

    def __str__(self):
        return self.name

    def avatar_url(self):
        return format_html(
            '<a href="{}"><img src="{}" height="30"></a>',
            self.avatar,
            self.avatar,
        )


    class Meta(object):
        verbose_name = _('wechat member')
        verbose_name_plural = _('wechat member')

    avatar_url.short_description= _('avatar')



from __future__ import unicode_literals
from future.standard_library import install_aliases
install_aliases()

from urllib.parse import quote
from wechat.api import Base

class Member(Base):
    """
    Member Class
    """

    def get_code_url(self, redirect_uri, state):
        """Get code url"""
        redirect_uri = quote(redirect_uri, safe='')
        # #becase error sorting...
        # url = 'https://open.weixin.qq.com/connect/oauth2/authorize'
        # param = {
        #     'appid': self.appid,
        #     'redirect_uri': redirect_uri,
        #     'response_type': 'code',
        #     'scope': 'snsapi_userinfo',
        #     'state': state,
        # }
        url = 'https://open.weixin.qq.com/connect/oauth2/authorize?appid=%s&redirect_uri=%s&response_type=code&scope=snsapi_userinfo&state=%s#wechat_redirect' % (self.appid, redirect_uri, state)
        #return self.get_url(url)
        return url

    def get_access_token_url(self, code):
        """Get access token url"""
        url = 'https://api.weixin.qq.com/sns/oauth2/access_token'
        param = {
            'appid': self.appid,
            'secret': self.appsecret,
            'code': code,
            'grant_type': 'authorization_code'
        }
        url = self.get_url(url, param)
        user = self.get_data(url)
        return user

    def get_user_info(self, code):
        """Get user base info"""
        user = self.get_access_token_url(code)
        url = 'https://api.weixin.qq.com/sns/userinfo'
        param = {
            'access_token': user['access_token'],
            'openid': user['openid'],
            'lang': 'zh_CN',
        }
        url = self.get_url(url, param)
        result = self.get_data(url)
        return result



from django.core.urlresolvers import reverse
from django.views.generic import View
from django.http import HttpResponseRedirect
from .models import Member
from .api import Member as MemberApi

def auth(request):
    wx = MemberApi()
    code = request.GET['code']
    state = request.GET['state']
    info = wx.get_user_info(code)

    try:
        member = Member.objects.get(openid=info['openid'])
    except Member.DoesNotExist:
        member = Member(
            name = info['nickname'],
            openid = info['openid'],
            avatar = info['headimgurl'],
            city = info['city'],
        )
    member.save()

    data = {
        "id": member.id,
        "openid": member.openid,
        "name": member.name,
        "avatar": member.avatar,
    }
    request.session['wx_member'] = data
    request.session['wx_member_id'] = member.id
    return HttpResponseRedirect(state)




class WxMemberView(View):
    """
    Wechat member base view
    """
    def dispatch(self, request, *args, **kwargs):
        try:
            member_id = request.session['wx_member_id']
            self.wx_member = Member.objects.get(id=member_id)
            return super(WxMemberView, self).dispatch(request, *args, **kwargs)
        except (KeyError, Member.DoesNotExist):
            return_uri = 'http://'\
                    + request.get_host()\
                    + reverse('wx_member:auth')

            wx = MemberApi()
            url = wx.get_code_url(return_uri, request.path)
            return HttpResponseRedirect(url)

